using System;
using System.IO;
using System.Text.RegularExpressions;
using System.Collections.Generic;

using PRoCon.Core;
using PRoCon.Core.Plugin;
using PRoCon.Core.Plugin.Commands;
using PRoCon.Core.Players;
using PRoCon.Core.Players.Items;


namespace PRoConEvents
{

	public class CMissingStuffCollector : PRoConPluginAPI, IPRoConPluginInterface
	{
		private bool fIsEnabled;
		private int fDebugLevel;
		private string hostname;
		private string port;
		private WeaponDictionary weapons;
		private SpecializationDictionary specializations;

		public CMissingStuffCollector()
		{
			fIsEnabled = false;
			fDebugLevel = 2;
		}

		public String GetPluginName()
		{
			return "Missing Stuff Collector";
		}

		public String GetPluginVersion()
		{
			return "0.0.0.1";
		}

		public String GetPluginAuthor()
		{
			return "MorpheusX(AUT)";
		}

		public String GetPluginWebsite()
		{
			return "https://myrcon.com";
		}

		public String GetPluginDescription()
		{
			return @"It collects missing stuff. Who would have thought? =)";
		}
	
		public List<CPluginVariable> GetDisplayPluginVariables()
		{
			List<CPluginVariable> lstReturn = new List<CPluginVariable>();

			lstReturn.Add(new CPluginVariable("Settings|Debug level", fDebugLevel.GetType(), fDebugLevel));

			return lstReturn;
		}

		public List<CPluginVariable> GetPluginVariables()
		{
			return GetDisplayPluginVariables();
		}

		public void SetPluginVariable(String strVariable, String strValue)
		{
			if (Regex.Match(strVariable, @"Debug level").Success) {
				int tmp = 2;
				int.TryParse(strValue, out tmp);
				fDebugLevel = tmp;
			}
		}


		public void OnPluginLoaded(String strHostName, String strPort, String strPRoConVersion)
		{
			this.RegisterEvents(this.GetType().Name, "OnPlayerKilled");
		
			this.hostname = strHostName;
			this.port = strPort;
		}

		public void OnPluginEnable()
		{
			fIsEnabled = true;
			ConsoleWrite("Enabled!");

			try
			{
				weapons = this.GetWeaponDefines();
				specializations = this.GetSpecializationDefines();

				string logFile = "Plugins\\missing_weapon_defines_" + this.hostname + "_" + this.port + ".txt";
				using (StreamWriter w = new StreamWriter(logFile, true))
				{
					w.WriteLine(DateTime.UtcNow);
					w.Flush();
				}
			}
			catch (Exception e)
			{
				ConsoleWrite(e.ToString());
			}
		}

		public void OnPluginDisable()
		{
			fIsEnabled = false;
			ConsoleWrite("Disabled!");
		}

		public override void OnPlayerKilled(Kill kKillerVictimDetails)
		{
			if (!this.weapons.Contains(kKillerVictimDetails.DamageType) && !this.specializations.Contains(kKillerVictimDetails.DamageType))
			{
				try
				{
					DebugWrite("Didn't contain " + kKillerVictimDetails.DamageType, 2);

					string logFile = string.Format("Plugins\\missing_weapon_defines_{0}_{1}.txt", this.hostname, this.port);
					string line = string.Empty;
					bool log = true;

					using (StreamReader r = new StreamReader(logFile))
					{
						while ((line = r.ReadLine()) != null)
						{
							if (string.Equals(line, kKillerVictimDetails.DamageType, StringComparison.OrdinalIgnoreCase))
							{
								DebugWrite("Found in log file (" + line + " == " + kKillerVictimDetails.DamageType + ")", 2);
								log = false;
							}
						}
					}

					if (log)
					{
						using (StreamWriter w = new StreamWriter(logFile, true))
						{
							w.WriteLine(kKillerVictimDetails.DamageType);
							w.Flush();
						}
					}
				}
				catch (Exception e)
				{
					ConsoleWrite(e.ToString());
				}
			}
		}

		public enum MessageType { Warning, Error, Exception, Normal };

		public String FormatMessage(String msg, MessageType type)
		{
			String prefix = "[^b" + GetPluginName() + "^n] ";

			if (type.Equals(MessageType.Warning))
				prefix += "^1^bWARNING^0^n: ";
			else if (type.Equals(MessageType.Error))
				prefix += "^1^bERROR^0^n: ";
			else if (type.Equals(MessageType.Exception))
				prefix += "^1^bEXCEPTION^0^n: ";

			return prefix + msg;
		}

		public void LogWrite(String msg)
		{
			this.ExecuteCommand("procon.protected.pluginconsole.write", msg);
		}

		public void ConsoleWrite(String msg, MessageType type)
		{
			LogWrite(FormatMessage(msg, type));
		}

		public void ConsoleWrite(String msg)
		{
			ConsoleWrite(msg, MessageType.Normal);
		}

		public void ConsoleWarn(String msg)
		{
			ConsoleWrite(msg, MessageType.Warning);
		}

		public void ConsoleError(String msg)
		{
			ConsoleWrite(msg, MessageType.Error);
		}

		public void ConsoleException(String msg)
		{
			ConsoleWrite(msg, MessageType.Exception);
		}

		public void DebugWrite(String msg, int level)
		{
			if (fDebugLevel >= level) ConsoleWrite(msg, MessageType.Normal);
		}

		public void ServerCommand(params String[] args)
		{
			List<String> list = new List<String>();
			list.Add("procon.protected.send");
			list.AddRange(args);
			this.ExecuteCommand(list.ToArray());
		}
	}
}



